C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine db_get_co2cld(cfgname,line,pixel,obs_clr,tcold,twarm,
     &               tdiff,pct,eca,qual_flag,conf_flag,hc_flag,ci_flag)

c!F77------------------------------------------------------------------
c!Description: Compute cloud top pressure and effective cloud amount 
c              from MODIS IR data.
c
c!Input parameters:
c cfgname                    Configuration file name
c line                       Line number within a swath (1-10)
c pixel                      Pixel number within a 1km scan (1-1500)
c obs_clr                    Observed clear sky pixels
c                              0=none present
c                              1=present
c                              use only with mod06ct_proc_opt = 2
c tcold                      Obs cloud or cold brightness temperatures
c                              (K) for MODIS IR bands 36, 35, 34, 33, 31
c twarm                      Obs clear or warm brightness temperatures
c                              (K) for MODIS IR bands 36, 35, 34, 33, 31
c tdiff                      Obs mean 8.5 - 11 um brightness temperature(K)
c
c The following in COMMON /MOD06_DATA/ are used:
c wprof1                     Water vapor mixing ratio profile (g/kg)
c tprof1                     Temperature profile (K)
c tem1                       Surface temperature (K)
c view                       Satellite zenith angle (deg)
c cs_bias_corr               clear-sky radiance bias correction 
c bad_value                  missing or bad data flag
c
c!Output parameters:
c qual_flag                  Product quality flag (0=not useful, 1=useful)
c conf_flag                  Product confidence flag (0-3, 0=worst, 3=best)
c hc_flag                    Indicates whether or not high cloud was
c                              detected (0=missing, 1=no, 2=yes)
c ci_flag                    Indicates whether or not cirrus cloud was
c                              detected (0=missing, 1=no, 2=yes)
c pct                        Cloud top pressure (hPa)
c eca                        Cloud effective emissivity
c
c The following are output to COMMON /MOD06_DATA/:
c height_tropopause          Pressure of tropopause from temperature profile
c cloud_h_method             Number of the selected ratio solution
c cloudtop_pre_ir            Window cloud top height (mb)
c spec_cloud_forcing         Cloud-clear or cold-warm radiance for MODIS 
c                              IR bands 36, 35, 34, 33, 31 (mW/m2/sr/cm-1)
c cloudtop_pres              Final cloud top height pressure (mb)
c cloudtop_temp              Cloud top temperature from temperature profile
c cloudtop_eff_emi           Effective cloud amount (no units)
c cloudtop_pres_from_ratios  Cloud top pressure for each of 5 ratios
c
c!Revision history:
c
c calls:
c function modrad       Compute MODIS IR TOA radiance given transmittance
c function modis_planck Compute Planck radiance for MODIS IR bands
c subroutine db_get_trans Compute transmittance profile for MODIS IR bands
c subroutine prof_int   Interpolates portion of input temperature, moisture,
c                          ozone, transmittance profiles
c
c!Team-unique Header:
c
c!End------------------------------------------------------------------

      implicit none
      save

      include 'db_mod06uw_data.inc'
      include 'db_mod06uw_debug.inc'

c ... scalar arguments
      real pct,eca,tdiff
      integer qual_flag,conf_flag,ci_flag,hc_flag,line,pixel,obs_clr

c ... array arguments
      character*(*) cfgname
      real tcold(ntbct),twarm(ntbct)

c ... local scalars
      real bot,top,db,fm,fm1,fm2,fmsav,ratio,rtemin,sum,
     &     tmin,tw,pfco2,ecawin,ecaco2,ptrp,pwin,tct,ts,theta
      integer id,is1,iw1,k,k1,k2,ll,ltrp,lwin,ip,mid_px,
     &        isp,lco2,ipco2,ngch,ipw,iout,jout,mid_ln,ii,jj
      logical process

c ... local arrays
      real amo(nsct),ra(pplev,nbct),rclr(nbct),robs(nbct),rterms(nsct),
     &     tau(plev,nbct),ozone(plev),pp(pplev),tp(pplev),
     &     taup(pplev,nbct),rmin(nbct),rwarm(nbct),rwcld(nsct),
     &     em_adj(nsct),delr(nbct),t(plev),w(plev),pco2(nsct),
     &     bias(nbct)
      integer lev(nsct),kch(nsct,2),mbnds(nbct),krto(nsct),mcsbnds(nbct)

c ... external functions
      real modis_planck, modrad
      external modis_planck, modrad

c ... external subroutines
      external db_get_trans, prof_int
      
c ... -----------------------------------------------------------------

c ... data statements

c ... Define indices of channel combinations used in the co2-slicing
c ... method.
      data kch/1,2,2,3,4,2,3,4,4,5/

c ... Zero-filled ozone profile passed to db_get_trans, which causes
c ... reference (US std atm 1976) ozone profile to be used.
      data ozone/plev*0.0/

c ... Order of MODIS IR band numbers used
      data mbnds/36,35,34,33,31/      

c ... Order of clear-sky radiance bias values used.
      data mcsbnds/5,4,3,2,1/

c ... Ice cloud emissivity adjustments for each channel combination.
      data em_adj/0.95,1.0,1.0,1.0,1.0/

c ... Define noise level (in radiance units) for each CO2 channel.
      data rmin /-2.5, -5.0, -20.0, -5.0, -0.5/

c ... -----------------------------------------------------------------

c ... Initialize flags
      do id = 1,nsct
        krto(id) = 0
        lev(id) = 0
      enddo

c ... Define output array indices for the current line and pixel.
      iout = (pixel / isamp) + 1
      jout = (line / isamp) + 1

c ... Define mid-point of box in terms of line and pixel.
      mid_px = pixel + isamp / 2
      mid_ln = line + isamp / 2

c ... -----------------------------------------------------------------

c ... Get temperature and water vapor mixing ratio profiles.
      do ii = 1,plev
        t(ii) = tprof1(ii,mid_px,mid_ln)
        w(ii) = wprof1(ii,mid_px,mid_ln)
      end do

c ... -----------------------------------------------------------------

c ... Get satellite zenith angle.
      theta = view(mid_px,mid_ln)

c ... Calculate transmittance profiles (now calls 42 level fast model)
      do k = 1,nbct
        call db_get_trans(cfgname,t,w,ozone,theta,k,tau)
      enddo

c ... Interpolate pressure, temperature, and transmittance profiles
c ... between levels corresponding to lbot and ltop.
      call prof_int(line,pixel,t,tau,pp,tp,taup,isp)

c ... -----------------------------------------------------------------
 
c ... Find level of coldest temperature between 100 mb and the surface.
      tmin = 350.0
      do ll = 20,isp
        if(tp(ll) .le. tmin) then
          tmin = tp(ll)
          ltrp = ll
        end if
      enddo
      ptrp = pp(ltrp)

c ... -----------------------------------------------------------------
  
c ... Compute the "window" cloud height.
      tw = tcold(kwc)
      do ll = ltrp,isp
        if(tp(ll) .lt. tw) then
          lwin = ll
        end if
      enddo

c ... Find which level tw is closest to and define beginning level at
c ... which to begin comparison of LHS and RHS of co2-slicing equation.
      if(lwin .eq. isp) then
        iw1 = lwin - 2
      else if( abs(tw - tp(lwin)) .gt. abs(tw - tp(lwin + 1)) ) then
        lwin = lwin + 1
        iw1 = lwin - 2
      else
        iw1 = lwin - 1
      end if

c ... Define all output values for window cloud height retrieval.
      pwin = pp(lwin)
      ecawin = 1.0
      ipw = 6
      pct = pwin
      eca = ecawin
      ip = ipw
      tct = tp(lwin)
      if(pwin .lt. 400.0) then
        hc_flag = 2
      else
        hc_flag = 1
      end if
      qual_flag = 1
      conf_flag = 1

c ... -----------------------------------------------------------------

c ... Fill all output arrays which pertain to the "window" cloud height
c ... retrieval. Some values may be overwritten below.
      cloud_h_method(iout,jout) = ip * 1.0
      cloudtop_pres(iout,jout) = pct
      cloudtop_temp(iout,jout) = tct
      height_tropopause(iout,jout) = ptrp
      cloudtop_eff_emi(iout,jout) = eca
      cloudtop_pre_ir(iout,jout) = pwin

c ... -----------------------------------------------------------------

c ... Write debug information about input data and IR "window" cloud
c ... height retrieval.
      if(debug .gt. 0) then
       if(line .eq. 6 .and. pixel .eq. 676) then
        write(h_output,'(1x)')
        write(h_output,'(''get_co2cld debug'')')
        write(h_output,'(''Line and pixel '',2i5)') line,pixel
        write(h_output,'(''Output array indices are '',2i10)')
     &      iout,jout
        write(h_output,'(''Mid-point of box is at '',2i10)')
     &      mid_ln,mid_px
        write(h_output,'(1x)')
        write(h_output,'(''Input data / window height retrieval'')')
        write(h_output,'(''Trop level and pressure '',i5,f10.3)')
     &      ltrp,ptrp
        write(h_output,'(''Surface pressure '',f10.3)') pp(isp)
        write(h_output,'(''Window cloud height level '',i4)') lwin
        write(h_output,'(''Window cloud height pressure '',
     &      ''and temperature '',2f10.3)') pwin,tct
       end if
      end if

c ... -----------------------------------------------------------------
  
c ... If using observed clear-sky radiances, make sure a valid clear-
c ... sky radiance was found.

      if( (mod06ct_proc_opt .ne. 2 ) .or.
     &    (mod06ct_proc_opt .eq. 2 .and. obs_clr .eq. 1) ) then

c ... -----------------------------------------------------------------
  
c ... Perform radiative transfer calculations for co2-slicing method.

c ... Get surface temperature.
      ts = tem1(mid_px,mid_ln)

c ... Define index of first level of integration in computing RHS of
c ... co2-slicing equation.
      is1 = isp - 1

      do k = 1,nbct
      
c ...   Compute TOA clear radiance from input profiles.
        rclr(k) = modrad(ts,tp,taup(1,k),mbnds(k),isp)

c ...   Get bias correction for this channel
        bias(k) = cs_bias_corr(mcsbnds(k),mid_px,mid_ln)

c ...   Get "warm" radiance value from either calculation or observations.
        if(mod06ct_proc_opt .eq. 1) then
c ...     Apply clear-sky radiance bias correction.
          if(bias(k) .ne. bad_value) then
            rwarm(k) = rclr(k) - bias(k)
          else
c ...       No correctionis available.
            rwarm(k) = rclr(k)
          end if
        else
          rwarm(k) = modis_planck(twarm(k),mbnds(k),0)
        end if

c ...   Get "cold" or "cloudy" radiance from observations.
        robs(k) = modis_planck(tcold(k),mbnds(k),0)

c ...   Calculate "cold - warm" or "clear - cloudy" value. These are 
c ...   components of the LHS of the co2-slicing equation.
        delr(k) = robs(k) - rwarm(k)
        spec_cloud_forcing(iout,jout,k) = delr(k)

c ...   Compute components of RHS 
        sum = 0.0
        do ll = is1,1,-1
          db = modis_planck(tp(ll + 1),mbnds(k),0)
     &             - modis_planck(tp(ll),mbnds(k),0)
          sum = sum + 0.5 * (taup(ll + 1,k) + taup(ll,k)) * db
          ra(ll,k) = sum
       enddo

      enddo

c ... -----------------------------------------------------------------
    
c ... Write debug information for radiative transfer calculations.
      if(debug .gt. 0) then
       if(line .eq. 6 .and. pixel .eq. 676) then
        write(h_output,'(1x)')
        write(h_output,'(''Radiative transfer section'')')
        write(h_output,'(''Surface temperature '',f10.3)') ts
        write(h_output,'(''Values for MODIS bands '',5i10)')
     &       (mbnds(ii),ii=1,nbct)
        write(h_output,'(''Calculated TOA radiances '',5f10.3)')
     &       (rclr(ii),ii=1,nbct)
        write(h_output,'(''Clear-sky radiance bias  '',5f10.3)')
     &       (bias(ii),ii=1,nbct)
        write(h_output,'(''Warm or clear radiances  '',5f10.3)')
     &       (rwarm(ii),ii=1,nbct)
        write(h_output,'(''Cold or cloudy radiances '',5f10.3)')
     &       (robs(ii),ii=1,nbct)
        write(h_output,'(''Spectral cloud forcing  '',5f10.3)')
     &       (delr(ii),ii=1,nbct)
       end if
      end if

c ... -----------------------------------------------------------------
  
c ... Check if 11 micron cold minus warm value is within instrument
c ... noise level. If not, cannot perform co2-slicing method.
      if(delr(kwc) .le. rmin(kwc)) then

        conf_flag = 2

c ...   Compute cloud height for each channel combination (when possible).
 
        do id = 1,nsct

          krto(id) = 0
          fmsav = 1000.0
          k1 = kch(id,1)
          k2 = kch(id,2)
c ...     Use the 11/13 um solution only for ice clouds (tdiff>2).
          if(id .ne. nsct .or. tdiff .ge. 2.0) then
            process = .true.
          else
            process = .false.
          end if

c ...     Check if values of cold minus warm are within instrument noise.
          if(delr(k1) .le. rmin(k1)) then
            if(delr(k2) .le. rmin(k2)) then
              if(process) then
      
c ...           Find minimum difference between LHS and RHS of co2-slicing
c ...           equation. Apply emissivity correction.
                do ll = iw1,ltrp,-1
                  fm1 = delr(k1) * ra(ll,k2)
                  fm2 = delr(k2) * ra(ll,k1)
                  fm = abs(fm1 - fm2 * em_adj(id))

                  if(fm .le. fmsav) then
                    fmsav = fm
                    lev(id) = ll
                  end if
                enddo

                if(fmsav .lt. 1000.0) then
                  krto(id) = 1
                end if

              end if
            end if
          end if

        enddo

        if(debug .gt. 0) then
         if(line .eq. 6 .and. pixel .eq. 676) then
          write(h_output,'(1x)')
          write(h_output,'(''Initial CO2-slicing cloud heights'')')
          write(h_output,'(''MODIS channel combinations:'')')
          write(h_output,'(8x,8(i8,''/'',i2))') (mbnds(kch(ii,1)),
     &       mbnds(kch(ii,2)),ii=1,nsct)
          write(h_output,'(''atm lev '',8i10)') (lev(ii),ii=1,nsct)
         end if
        end if

c ...   ---------------------------------------------------------------
  
c ...   Compute effective cloud amounts for every channel combination
c ...   for which there was a successful cloud height retrieval.
        do id = 1,nsct
          rwcld(id) = bad_value
          amo(id) = bad_value
          if(krto(id) .ne. 0) then
            ll = lev(id)
            rwcld(id) = modis_planck(tp(ll),mbnds(kwc),0)
            bot = rwcld(id) - rclr(kwc)
            top = delr(kwc)
            if(abs(bot) .gt. 0.1) then
              ratio = top / bot
              if(ratio .le. 1.0 .and. ratio .ge. 0.01) then
                amo(id) = ratio
              else
                krto(id) = 0
              end if
            else
              krto(id) = 0
            end if
          end if
        enddo
     
c ...   ---------------------------------------------------------------
	  
c ...   Write debug information about effective emissivity calculations
        if(debug .gt. 0) then
         if(line .eq. 6 .and. pixel .eq. 676) then
          write(h_output,'(1x)')
          write(h_output,'(''Initial CO2-slicing eff. emiss.'')')
          write(h_output,'(''MODIS channel combinations:'')')
          write(h_output,'(7x,8(i8,''/'',i2))') (mbnds(kch(ii,1)),
     &       mbnds(kch(ii,2)),ii=1,nsct)
          write(h_output,'(''cld rad   '',8f10.3)')
     &       (rwcld(ii),ii=1,nsct)
          write(h_output,'(''clr rad, delr '',2f10.3)') rclr(kwc),
     &       delr(kwc)
          write(h_output,'(''eff emiss '',8f10.3)')
     &       (amo(ii),ii=1,nsct)
         end if
        end if
  
c ...   ---------------------------------------------------------------
  
c ...   Find the cloud height that best satisfies the radiative transfer.
        do id = 1,nsct
          if(krto(id) .ne. 0) then
            rterms(id) = 0.0
            ll = lev(id)
            do k = 1,nbct
              rterms(id) = rterms(id) + (robs(k) - rclr(k) -
     &                          amo(id) * ra(ll,k)) **2
            enddo
          else
            rterms(id) = bad_value
          end if
        enddo

c ...   ---------------------------------------------------------------
  
c ...   Find the cloud height which gave the smallest error.
        ngch = 0
        rtemin = 999999.9
        do id = 1,nsct
          if(krto(id) .ne. 0) then
c ...       Count the number of valid cloud heights.
            ngch = ngch + 1
            if(rterms(id) .lt. rtemin) then
              rtemin = rterms(id)
              lco2 = lev(id)
              ipco2 = id
            end if
c ...       Save individual cloud height values for output.
            pco2(id) = pp(lev(id))
            cloudtop_pres_from_ratios(iout,jout,id) = pco2(id)
          end if
        enddo

c ...   ---------------------------------------------------------------

c ...   Write debug information about final cloud height selection.
        if(debug .gt. 0) then
         if(line .eq. 6 .and. pixel .eq. 676) then
          write(h_output,'(1x)')
          write(h_output,'(''Cloud height selection'')')
          write(h_output,'(''MODIS channel combinations:'')')
          write(h_output,'(8x,8(i7,''/'',i2))') (mbnds(kch(ii,1)),
     &       mbnds(kch(ii,2)),ii=1,nsct)
          write(h_output,'(''heights '',8f12.3)') (pco2(ii),ii=1,nsct)
          write(h_output,'(''eff emiss '',8f10.3)') (amo(ii),ii=1,nsct)
          write(h_output,'(''errors    '',8f10.3)')
     &       (rterms(ii),ii=1,nsct)
         end if
        end if

c ...   ---------------------------------------------------------------
  
c ...   Check if there were any valid cloud heights.
        if(ngch .ne. 0) then

c ...     Save best cloud height for output.
          pfco2 = pp(lco2)
          ecaco2 = amo(ipco2)

c ...     Check for any conditions which may invalidate final co2-slicing
c ...     cloud height.
          if(pfco2 .gt. 700.0 .and. ecaco2 .le. 0.95) then

c ...       Do not output co2-slicing result. The window cloud height
c ...       (calculated above) will be reported instead.

          else

c ...       Prepare final cloud height for output and set appropritate
c ...       flags.
            pct = pfco2
            eca = ecaco2
            ip = ipco2
            tct = tp(lco2)
            conf_flag = 3
            if(eca .le. 0.95) then
              ci_flag = 2
            else
              ci_flag = 1
            end if
            if(pct .lt. 400.0) then
              hc_flag = 2
            else
              hc_flag = 1
            end if

c ...       Fill output arrays.
            cloud_h_method(iout,jout) = ip * 1.0
            cloudtop_pres(iout,jout) = pct
            cloudtop_temp(iout,jout) = tct
            cloudtop_eff_emi(iout,jout) = eca

          end if
        end if
  
c ...   ---------------------------------------------------------------
  
      end if

      end if

c ... Write output debug information
      if(debug .gt. 0) then
       if(line .eq. 6 .and. pixel .eq. 676) then
        write(h_output,'(1x)')
        write(h_output,'(''Output from get_co2cld'')')
        write(h_output,'(''Tropopause height '',f10.3)') 
     &    height_tropopause(iout,jout)
        write(h_output,'(''Window cloud height '',f10.3)') 
     &    cloudtop_pre_ir(iout,jout)
        write(h_output,'(''Final CO2 cloud height '',f10.3)') 
     &    cloudtop_pres(iout,jout)
        write(h_output,'(''Cloud effective emissivity '',f10.3)') 
     &    cloudtop_eff_emi(iout,jout)
        write(h_output,'(''Cloud temperature '',f10.3)') 
     &    cloudtop_temp(iout,jout)
        write(h_output,'(''CO2 cloud height solutions '',8f10.3)') 
     &    (cloudtop_pres_from_ratios(iout,jout,jj),jj=1,nsct)
        write(h_output,'(''CO2 spectral cloud forcings'',8f10.3)') 
     &    (spec_cloud_forcing(iout,jout,jj),jj=1,nsct)
        write(h_output,'(''Index of solution chosen'',f10.3)') 
     &    cloud_h_method(iout,jout)
        write(h_output,'(1x)')
        write(h_output,'(''Flag values'')')
        write(h_output,'(''qual_flag,conf_flag,hc_flag,ci_flag '',
     &    4i5)') qual_flag,conf_flag,hc_flag,ci_flag
       end if
      end if

c ... -----------------------------------------------------------------
  
      return
      end
