C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine db_mod06_check_box( line, pixel, nmissing, ncloud,
     &           ngood_msk, ngood_co2, ngood_IRphase, ngood_dayIRphase )

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c    Check a 5x5 sampling box to see if there is enough good data
c    to perform a MOD06 retrieval.  This version is used with the 
c    Driect Broadcast version of the Cloud Top Properties MOD06 product.
c
c!Input Parameters:
c    LINE        Line number within a 1km scan (1-10)
c    PIXEL       Pixel number within a 1km scan (1-1500)
c
c!Output Parameters:
c    NGOOD       Number of good pixels found within box
c                (where BOX_FLAG=1)
c    NMISSING    Number of missing pixels within box (bad radiances)
c    NCLOUD      Number of cloudy pixels within box
c    NGOOD_MSK   Number of good pixels as defined by bit 1 of cloud mask
c    NGOOD_CO2   Number of good pixels as defined by bit 1 of cloud mask
c                   for CO2-slicing algorithm
c    NGOOD_IRPHASE Number of good pixels as defined by bit 1 of cloud 
c                   mask for IR Phasse algorithm
c    NGOOD_dayIRPHASE Number of good pixels as defined by bit 1 of 
c                   cloud mask for daytime IR Phasse algorithm
c    The following array in COMMON /MOD06_DATA/ is filled:
c    BOX_FLAG    Valid data flag array for box
c                (0 = Bad, 1 = Good)
c                Good data implies that:
c                - Radiances are > 0.0 in all MODIS IR bands,
c                - Geolocation data are valid,
c                - Ancillary data are valid,
c                - Cloud mask was determined.
c    BOX_CLOUD   Clear/cloud flag array for box
c                (-1 = undefined, 0 = Clear, 1 = Cloud)
c
c!Revision History:
c
c!Team-unique Header:
c
c!End
c
c-----------------------------------------------------------------------

      implicit none

      
      include 'db_mod06uw_data.inc'
      include 'db_mod06uw_debug.inc'
      
c ... arguments

      integer line, pixel, nmissing, ncloud, ngood_msk
      integer ngood_IRphase, ngood_dayIRphase, ngood_co2
      integer nday, nglint

c ... local variables

      integer i, j, k, ibox, jbox
      byte mask_byte, mask_test, mask_value

c ... save statement

      save / mod06_data /, / mod06_debug /

c ... Set default return values

      nmissing    = 0
      ncloud      = 0
      ngood_msk   = 0
c ... some additions by Baum 8-7-2000
      ngood_IRphase    = 0
      ngood_dayIRphase = 0
      ngood_co2        = 0
      
c ... Number of  pixels in box in daylight; based on dn_flag
      nday   = 0  
c ... Number of pixels in box identified as containing sunglint
      nglint = 0  
      
      do jbox = 1, isamp
        do ibox = 1, isamp
          box_flag( ibox, jbox )       = 1
          box_cloud( ibox, jbox )      = -1
          box_flag_IR( ibox, jbox )    = 1
          box_flag_co2( ibox, jbox )   = 1
          box_flag_dayIR( ibox, jbox ) = 1
        end do
      end do

c ... Start loop over all pixels in this box

      jbox = 1
      do j = line, line + isamp - 1
        ibox = 1
        do i = pixel, pixel + isamp - 1

c ...     Check that cloud mask was determined for this pixel, and if so
c ...     check cloud mask
c ...     (99% and 95% confidence clear pixels are classified CLEAR)
c ...     (Low confidence clear and cloudy pixels are classified CLOUD)

          mask_byte = mask1(  i, j )
          mask_test = 1
          mask_value = iand( mask_byte, mask_test )
          if ( mask_value .eq. 1 ) then
            ngood_msk = ngood_msk + 1
            mask_test = 6
            mask_value = iand( mask_byte, mask_test )
            if ( mask_value .ge. 4 ) then
              box_cloud( ibox, jbox ) = 0
            else
              box_cloud( ibox, jbox ) = 1
            endif
          endif
  
c ...     Now check sunglint flag in cloud mask
          mask_byte = mask1( i, j )
          mask_test = 1
          mask_value = iand( mask_byte, mask_test )
          if ( mask_value .eq. 1 ) then
            mask_test = 16
            mask_value = iand( mask_byte, mask_test )
            if ( mask_value .lt. 16 ) nglint = nglint + 1
          endif

c ...     Check for valid radiance data for IR-only phase algorithm

          do k = 1, irphase_band
            if ( radiance1( i, j, ir_aband(k) ) .le. 0.0 ) then
              box_flag_IR( ibox, jbox ) = 0
            endif
          end do
          if (box_flag_IR(ibox,jbox) .eq. 0) nmissing = nmissing + 1


c ...     Check for valid radiance data for CO2 slicing algorithm

          do k = 1, co2_band
            if ( radiance1( i, j, ir_co2bandptr(k) ) .le. 0.0 ) then
              box_flag_co2( ibox, jbox ) = 0
            endif
          end do


c ...     Check for valid radiance data for daytime cloud phase 
c ...      algorithm

          do k = 1, dayphase_band
            if ( radiance1( i, j, day_bandptr(k) ) .le. 0.0 ) then
              box_flag_dayIR( ibox, jbox ) = 0
            endif
          end do
 
c ...     Check ancillary data

          if ( tem1( i, j ) .le. 0.0 ) box_flag( ibox, jbox ) = 0
          if ( wat1( i, j ) .le. 0.0 ) box_flag( ibox, jbox ) = 0
          if ( pre1( i, j ) .le. 0.0 ) box_flag( ibox, jbox ) = 0
          do k = 1, 40
            if ( tprof1( k, i, j ) .le. 0.0 ) box_flag( ibox, jbox ) = 0
            if ( wprof1( k, i, j ) .le. 0.0 ) box_flag( ibox, jbox ) = 0
          end do

c ...     End loop over all pixels in this box

          ibox = ibox + 1
        end do
        jbox = jbox + 1
      end do

c ... Count Good/Bad and Clear/Cloudy pixels in this box

        do jbox = 1, isamp
          do ibox = 1, isamp
           if (box_flag_co2(ibox,jbox) .eq. 1 .and. 
     +             box_flag(ibox,jbox)  .eq. 1) then
                ngood_co2 = ngood_co2 + 1
            endif
c ...       Indicates both IR data and Ancillary data are good
            ngood_IRphase = ngood_IRphase + box_flag_IR( ibox, jbox )
            ngood_dayIRphase = ngood_dayIRphase + 
     +           box_flag_dayIR(ibox, jbox)
            ncloud = ncloud + box_cloud( ibox, jbox )
c ...       Indicates this is a day or night pixel 
            if (dn_flag(ibox,jbox) .eq. 1) nday=nday+1
          end do
        end do

c ... Write debug info for nadir box

      if ( debug .gt. 0 ) then
        if ( line .eq. 6 .and. pixel .eq. 676 ) then

          write( h_output, '(/,a,/)' ) 'mod06_check_box debug'

          write( h_output,
     &      '(''Found'',i4,'' Missing Pixels for Nadir Box'')') nmissing

          write( h_output,
     &      '(''Found'',i4,'' Cloudy Pixels for Nadir Box'')') ncloud
     
          write( h_output,
     &      '(''Found'',2i4,'' Good Data Pixels for Nadir Box'')') 
     &          ngood_co2, ngood_IRphase

        endif
      endif

      end
