C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine extend_profile( ni, pres, temp, mixr, lat, tf, wf )

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c    Interpolate the NCEP GDAS1 temperature and moisture profiles to
c    the 40 standard pressure levels required for CIMSS transmittance
c    models.
c
c!Input Parameters:
c    NI      Number of input GDAS1 pressure levels
c    PRES    GDAS1 pressure levels (hPa) from maximum to minimum
c    TEMP    GDAS1 temperature (K) at PRES levels
c    MIXR    GDAS1 water vapor mixing ratio (g/kg) at PRES levels
c    LAT     Latitude (deg) of GDAS1 data
c
c!Output Parameters:
c    TF      Temperature (K) at 40 standard levels
c    WF      Water vapor mixing ratio (g/kg) at 40 standard levels
c            (NOTE: see variable PF for pressures at 40 standard levels)
c
c!Revision History:
c
c!Team-unique Header:
c
c!End
c
c-----------------------------------------------------------------------

      implicit none

c ... arguments
            
      integer ni
      real pres( ni ), temp( ni ), mixr( ni ), lat, tf( 40 ), wf( 40 )
      
c ... local variables

      integer k, nf
      parameter ( nf = 40 )
      real pref( nf ), lnpf( nf ), pi( 1000 ), lnpi( 1000 ), ti( 1000 ),
     &  wi( 1000 )

c ... data statements

c ... 40 standard CIMSS pressure levels (hPa)

      data pref /
     &  .1,.2,.5,1.,1.5,2.,3.,4.,5.,7.,10.,15.,20.,25.,30.,
     &  50.,60.,70.,85.,100.,115.,135.,150.,200.,250.,300.,350.,400.,
     &  430.,475.,500.,570.,620.,670.,700.,780.,850.,920.,950.,1000. /

c ... Reverse the order of the input p, t, w arrays
      
      do k = 1, ni
        pi( k ) = pres( ni - k + 1 )
        ti( k ) = temp( ni - k + 1 )
        wi( k ) = mixr( ni - k + 1 )
      end do

c ... Convert pressure to ln(pressure)

      do k = 1, ni
        lnpi( k ) = log( pi( k ) )
      end do
      do k = 1, nf
        lnpf( k ) = log( pref( k ) )
      end do
      
c ... Interpolate to the 40 standard levels

      call interp( ni, lnpi, ti, nf, lnpf, tf )
      call interp( ni, lnpi, wi, nf, lnpf, wf )

c ... Extend temperatures from top of input profile to
c ... top of output profile
      
      do k = 1, nf
        if ( pref( k ) .lt. pi( 1 ) ) tf( k ) = -1.0
      end do
      call extemp( tf, lat )
      
      end
