C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      REAL FUNCTION MODIS_BRIGHT(RAD, BAND, UNITS)

C-----------------------------------------------------------------------
C!F77
C
C!DESCRIPTION:
C    Compute brightness temperature for a MODIS infrared band
C    on Terra or Aqua.
C
C    Spectral responses for each IR detector were obtained from MCST:
C    ftp://mcstftp.gsfc.nasa.gov/pub/permanent/MCST in
C    directories PFM_L1B_LUT_4-30-99 and FM1_RSR_LUT_07-10-01
C
C    An average spectral response for each infrared band was
C    computed. The band-averaged spectral response data were used
C    to compute the effective central wavenumbers and temperature
C    correction coefficients included in this module.
C
C    NOTE: The plaform name ('Terra' or 'Aqua') is passed to this
C    function via the common block defined in 'platform_name.inc'.
C
C!INPUT PARAMETERS:
C    RAD (REAL)      Planck radiance (units are determined by UNITS)
C    BAND (LONG)     MODIS IR band number (20-25, 27-36)
C    UNITS (LONG)    Flag defining radiance units
C                    0 => milliWatts per square meter per
C                         steradian per inverse centimeter
C                    1 => Watts per square meter per
C                         steradian per micron
C
C!OUTPUT PARAMETERS:
C    MODIS_BRIGHT  Brightness temperature (Kelvin)
C                  Note that a value of -1.0 is returned if
C                  RAD .LE. 0.0, or BAND is not in range 20-25, 27-36.
C
C!REVISION HISTORY:
C    Liam.Gumley@ssec.wisc.edu
C
C!TEAM-UNIQUE HEADER:
C    Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
C
C!END
C-----------------------------------------------------------------------

      IMPLICIT NONE

c ... Include files
      include 'platform_name.inc'
      
c ... Arguments
      real rad
      integer band, units

c ... Local variables
      real cwn_terra(16), tcs_terra(16), tci_terra(16)
      real cwn_aqua(16), tcs_aqua(16), tci_aqua(16)
      real cwn, tcs, tci
      integer index

c ... External functions
      real bright_m, brite_m
      external bright_m, brite_m
            
c ... Data statements

c-----------------------------------------------------------------------
c     BAND-AVERAGED MODIS SPECTRAL RESPONSE FUNCTIONS FOR TERRA
c     (LG 08-JAN-2002)
c     TEMPERATURE RANGE FOR FIT WAS  180.00 K TO  320.00 K
c
c     BANDS
c      20,  21,  22,  23,
c      24,  25,  27,  28,
c      29,  30,  31,  32,
c      33,  34,  35,  36,

c ... Effective central wavenumbers (inverse centimeters)
      data cwn_terra/
     & 2.641775E+03, 2.505277E+03, 2.518028E+03, 2.465428E+03,
     & 2.235815E+03, 2.200346E+03, 1.477967E+03, 1.362737E+03,
     & 1.173190E+03, 1.027715E+03, 9.080884E+02, 8.315399E+02,
     & 7.483394E+02, 7.308963E+02, 7.188681E+02, 7.045367E+02/

c ... Temperature correction slopes (no units)
      data tcs_terra/ 
     &  9.993411E-01, 9.998646E-01, 9.998585E-01, 9.998682E-01,
     &  9.998820E-01, 9.998845E-01, 9.994878E-01, 9.994918E-01,
     &  9.995496E-01, 9.997399E-01, 9.995607E-01, 9.997256E-01,
     &  9.999161E-01, 9.999167E-01, 9.999192E-01, 9.999282E-01/

c ... Temperature correction intercepts (Kelvin)
      data tci_terra/
     &  4.770522E-01, 9.264053E-02, 9.756834E-02, 8.928794E-02,
     &  7.309468E-02, 7.061112E-02, 2.204659E-01, 2.045902E-01,
     &  1.599076E-01, 8.249532E-02, 1.302885E-01, 7.181662E-02,
     &  1.970605E-02, 1.912743E-02, 1.816222E-02, 1.579983E-02/
c-----------------------------------------------------------------------
c     BAND-AVERAGED MODIS SPECTRAL RESPONSE FUNCTIONS FOR AQUA
c     (LG 08-JAN-2002)
c     TEMPERATURE RANGE FOR FIT WAS  180.00 K TO  320.00 K
c
c     BANDS
c      20,  21,  22,  23,
c      24,  25,  27,  28,
c      29,  30,  31,  32,
c      33,  34,  35,  36,

c ... Effective central wavenumbers (inverse centimeters)
      data cwn_aqua/
     &  2.647409E+03, 2.511760E+03, 2.517908E+03, 2.462442E+03,
     &  2.248296E+03, 2.209547E+03, 1.474262E+03, 1.361626E+03,
     &  1.169626E+03, 1.028740E+03, 9.076813E+02, 8.308411E+02,
     &  7.482978E+02, 7.307766E+02, 7.182094E+02, 7.035007E+02/

c ... Temperature correction slopes (no units)
      data tcs_aqua/ 
     &  9.993363E-01, 9.998626E-01, 9.998627E-01, 9.998707E-01,
     &  9.998737E-01, 9.998770E-01, 9.995694E-01, 9.994867E-01,
     &  9.995270E-01, 9.997382E-01, 9.995270E-01, 9.997271E-01,
     &  9.999173E-01, 9.999070E-01, 9.999198E-01, 9.999233E-01/

c ... Temperature correction intercepts (Kelvin)
      data tci_aqua/
     &  4.818401E-01, 9.426663E-02, 9.458604E-02, 8.736613E-02,
     &  7.873285E-02, 7.550804E-02, 1.848769E-01, 2.064384E-01,
     &  1.674982E-01, 8.304364E-02, 1.343433E-01, 7.135051E-02,
     &  1.948513E-02, 2.131043E-02, 1.804156E-02, 1.683156E-02/
c-----------------------------------------------------------------------

c ... Set default return value
      modis_bright = -1.0

c ... Check input parameters and return if they are bad
      if (rad .le. 0.0 .or.
     &    band .lt. 20 .or.
     &    band .gt. 36 .or.
     &    band .eq. 26) return

c ... Get index into coefficient arrays
      if (band .le. 25) then
        index = band - 19
      else
        index = band - 20
      endif
      
c ... Get the coefficients for Terra or Aqua
      if (platform_name(1:5) .eq. 'Terra' .or.
     &    platform_name(1:5) .eq. 'terra' .or.
     &    platform_name(1:5) .eq. 'TERRA') then
        cwn = cwn_terra(index)
        tcs = tcs_terra(index)
        tci = tci_terra(index)
      else if (platform_name(1:4) .eq. 'Aqua' .or.
     &         platform_name(1:4) .eq. 'aqua' .or.
     &         platform_name(1:4) .eq. 'AQUA') then
        cwn = cwn_aqua(index)
        tcs = tcs_aqua(index)
        tci = tci_aqua(index)
      else
        call message('modis_bright.f',
     &    'Platform name not recognized ' ,
     &     0, 2)
      endif
     
c ... Compute brightness temperature
      if (units .eq. 1) then

c ...   Radiance units are
c ...   Watts per square meter per steradian per micron
        modis_bright = (bright_m(1.0e+4 / cwn, rad) - tci) / tcs

      else

c ...   Radiance units are
c ...   milliWatts per square meter per steradian per wavenumber
        modis_bright = (brite_m(cwn, rad) - tci) / tcs

      endif

      END
