C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine prof_int(line,pixel,t,tau,pp,tp,taup,isp)

c!F77------------------------------------------------------------------
c!Description: Interpolates values of pressure, temperature, and 
c              transmittance between the levels indicated by lbot and 
c              ltop. Also determines the surface pressure level in the
c              output pressure profile.
c
c!Input parameters:
c integer          line       Line in scan (1-10)
c integer          pixel      Pixel in line (1-1500)
c real             t          Original temperature profile
c real             tau        Original transmittance profile
c
c!Output parameters:
c real             pp         Interpolated pressure profile
c real             tp         Interpolated temperature profile (K)
c real             taup       Interpolated transmittance profile
c integer          isp        Pressure level of surface
c
c!Revision history:
c
c Initial revision
c
c!Team-unique Header:
c!End------------------------------------------------------------------

      implicit none
      save

      include 'db_mod06uw_data.inc'
      include 'db_mod06uw_debug.inc'

c     Scalar arguments
      integer line,pixel,isp

c     Array arguments 
      real t(plev),tau(plev,nbct),pp(pplev),tp(pplev),taup(pplev,nbct)

c     Local scalars
      real dist,delta,psfc
      integer ilev,i,j,jj,n,mid_px,mid_ln
      logical looking

c     Local arrays
      real p(plev)

c ... -----------------------------------------------------------------

c     Data statements
c     Distance between levels to interpolate values
      data dist /0.5/

c ... Define input atmospheric pressure levels (40).
      data p /.1, .2, .5, 1., 1.5, 2., 3., 4., 5., 7., 10., 15., 20.,
     &        25., 30., 50., 60., 70., 85., 100., 115., 135., 150.,
     &        200., 250., 300., 350., 400., 430., 475., 500., 570.,
     &        620., 670., 700., 780., 850., 920., 950., 1000./
 
c ... -----------------------------------------------------------------

c ... Define mid-point of box in terms of line and pixel.
      mid_px = pixel + isamp / 2
      mid_ln = line + isamp / 2

c ... Get surface pressure.
      psfc = pre1(mid_px,mid_ln)

c ... -----------------------------------------------------------------

      ilev = 0
      looking = .true.

      do j = 1,plev

        if(j .gt. ltop .and. j .le. lbot) then

          ilev = ilev + 1
          delta = p(j) - p(j-1)
          pp(ilev) = dist * delta + p(j-1)
          delta = t(j) - t(j-1)
          tp(ilev) = dist * delta + t(j-1)
          do n = 1,nbct
            delta = tau(j,n) - tau(j-1,n)
            taup(ilev,n) = dist * delta + tau(j-1,n)
          enddo
          if(looking .and. psfc .le. pp(ilev)) then
            looking = .false.
            isp = ilev
          end if

        end if

        ilev = ilev + 1
        pp(ilev) = p(j)
        tp(ilev) = t(j)
        do n = 1,nbct
          taup(ilev,n) = tau(j,n)
        enddo
        if(looking .and. psfc .le. pp(ilev)) then
          looking = .false.
          isp = ilev
        end if

      enddo

      if(looking) then
        isp = ilev
      end if

c ... -----------------------------------------------------------------

c ... Write debug information.
      if(debug .gt. 0) then
       if(line .eq. 6 .and. pixel .eq. 676) then
        write(h_output,'(1x)')
        write(h_output,'(''Subroutine prof_int'')')
        write(h_output,'(''Input profiles:'')')
        write(h_output,'(''Atm level, pressure, temperature, '',
     &    ''transmittances'')')
        do i=1,plev
         write(h_output,'(i4,7f10.4)') i,p(i),t(i),(tau(i,jj),
     &     jj=1,nbct)
        enddo
        write(h_output,'(1x)')
        write(h_output,'(''Output profiles:'')')
        write(h_output,'(''Atm level, pressure, temperature, '',
     &    ''transmittances'')')
        do i=1,ilev
         write(h_output,'(i4,7f10.4)') i,pp(i),tp(i),(taup(i,jj),
     &     jj=1,nbct)
        enddo
        write(h_output,'(''Surface pressure, atm level '',f8.2,i7)')
     &    psfc,isp
       end if
      end if

c ... -----------------------------------------------------------------

      return
      end
