C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine read_goode( nx, ny, lat, lon, lun, result )

c-----------------------------------------------------------------------
c !F77
c
c !DESCRIPTION:
c     Given a set of lat/lon points, read an Interrupted Goode
c     Homolosine dataset from USGS of the type described at
c     http://edcwww.cr.usgs.gov/landdaac/glcc/globe_int.html and
c     http://edcwww.cr.usgs.gov/landdaac/1KM/goodesarticle.html
c
c     The data dimensions of the Interrupted Goode Homolosine
c     projection for the global land cover characteristics data set are
c     17347 lines (rows) and 40031 samples (columns).
c
c     Files of this type are exactly 17347 x 40031 = 694417757 bytes
c     in size, and files in use at UW include goge1_2_img, lst1km.v3.
c
c !INPUT PARAMETERS:
c     NX      First dimension of arrays LAT, LON, RESULT
c     NY      Second dimension of arrays LAT, LON, RESULT
c     LAT     Array of latitude values (degrees)
c     LON     Array of longitude values (degrees, -180W to +180E)
c     LUN     Logical unit number for input file
c             (must be opened by the caller for direct access with
c              a record length of 512 bytes. Caller must close the file).
c           
c !OUTPUT PARAMETERS:
c     RESULT  Array of land type values
c
c !REVISION HISTORY:
c
c !TEAM-UNIQUE HEADER:
c     Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
c
c !END
c-----------------------------------------------------------------------
      
      implicit none
      save

c --- parametes
      integer		nx
      integer		ny
      real 		lat(nx,ny)
      real		lon(nx,ny)
      integer		lun
      byte 		result(nx,ny)

c --- constants
      integer 		max_row
      parameter         (max_row = 17347)
      integer		max_col
      parameter         (max_col = 40031)

      integer		recsize
      parameter		(recsize = 512)

      integer		max_count
      parameter		(max_count = 3000000)
      
      integer		max_rec
      parameter		(max_rec = (max_row*max_col )/recsize)

c --- internal variables
      byte 		recdata( recsize )
      double precision  dlat
      double precision  dlon
      double precision  drow
      double precision  dcol
      integer	        max_pixel
      integer	        max_line
      integer	        pixel
      integer	        line
      integer	        row
      integer	        col
      integer		loc
      integer		count
      integer		kflag
      integer		i
      integer		recnum
      integer		elenum
      integer		oldrec
      integer		ios
      integer		rectest
      integer		loc_value( max_count )
      integer		loc_index( max_count )
      logical 		opentest
      

c CHECK ARGUMENTS

c --- Check that input arrays do not exceed local work array size
      if ( nx * ny .gt. max_count ) then
        call message( 'read_goode.f',
     &    'NX*NY greater than MAX_COUNT ', 0, 3 ) 
      endif

c --- Check that input LUN is open
      INQUIRE( lun, opened=opentest )
      if ( .not. opentest ) then
        call message( 'read_goode.f',
     &    'Input file LUN is not open ' , 0, 3 )
      endif
      
c --- Check that input LUN was opened with the correct record size
      INQUIRE( lun, recl=rectest )
      if ( rectest .ne. recsize ) then
        call message( 'read_goode.f',
     &    'Record size of input file does not match RECSIZE ', 0, 3)
      endif
 
     
c COMPUTE BYTE LOCATIONS FOR ALL LINES/PIXELS

c --- Begin loop over all lines and pixels
      max_pixel = nx
      max_line  = ny
      count = 0
      
      do line = 1, max_line
      do pixel = 1, max_pixel

c ------ Increment pixel count
          count = count + 1

c ------ Compute row/column using USGS routine (original version from
c        ftp://edcftp.cr.usgs.gov/pub/software/misc/gihll2ls.c)

         dlat = dble( lat( pixel, line ) )
         dlon = dble( lon( pixel, line ) )
         call getcoord( dlat, dlon, drow, dcol )
         row = nint( drow )
         col = nint( dcol )
          
c ------ Constrain row/column to allowed range
         row = max( min( row, max_row ), 1 )
         col = max( min( col, max_col ), 1 )

c ------ Compute byte location
         loc = ( row - 1 ) * max_col + col

c ------ Save location value and index
         loc_value( count ) = loc
         loc_index( count ) = count

      enddo
      enddo

c --- Sort the location value array in increasing order,
c     and re-order the index array the same way
      kflag = 2
      call isort( loc_value, loc_index, count, kflag )


c READ DATA AT SORTED BYTE LOCATIONS

c --- Loop through sorted byte locations
      oldrec = 0
      do i = 1, count
      
c ------ Compute record number and element number within record,
c        assuming the first record in the file is record one,
c        and the first element in each record is element one.
         recnum = ( loc_value( i ) - 1 ) / recsize + 1
         elenum = mod( loc_value( i ) - 1, recsize ) + 1

c ------ Constrain record number to allowed range
c        (note that this only has an effect on the bottom right hand
c        side of the projection, where all data values are 'ocean'.
c        See http://edcwww.cr.usgs.gov/landdaac/1KM/FIG4.gif).
         row = max( min( recnum, max_rec ), 1 )

c ------ If record number is new, read the data
         if ( recnum .ne. oldrec ) then

          READ( 
     &          UNIT=lun, 
     &          REC=recnum, 
     &          IOSTAT=ios 
     &        ) recdata
          if ( ios .ne. 0 ) then
            call message( 'read_goode.f',
     &        'Error reading input file: I/O status= ', ios, 3)
          endif
          
          oldrec = recnum

        endif

c ------ Compute the line and pixel number
         line  = ( loc_index( i ) - 1 ) / max_pixel + 1
         pixel = mod( loc_index( i ) - 1, max_pixel ) + 1

c ------ Store the data for this pixel/line
         result( pixel, line ) = recdata( elenum )
                    
      enddo

      end
